@@ -1,8 +1,7 @@
 /******/ (() => { // webpackBootstrap
 /******/ 	"use strict";
-var __webpack_exports__ = {};
 
-;// CONCATENATED MODULE: ./modules/e2ee/crypto-utils.js
+;// ./modules/e2ee/crypto-utils.js
 /**
  * Derives a set of keys from the master key.
  * @param {CryptoKey} material - master key to derive from
@@ -41,7 +40,7 @@ async function ratchet(material) {
     const textEncoder = new TextEncoder();
 
     // https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/deriveBits
-    return crypto.subtle.deriveBits({
+    return await crypto.subtle.deriveBits({
         name: 'HKDF',
         salt: textEncoder.encode('JFrameRatchetKey'),
         hash: 'SHA-256',
@@ -58,10 +57,10 @@ async function ratchet(material) {
  */
 async function importKey(keyBytes) {
     // https://developer.mozilla.org/en-US/docs/Web/API/SubtleCrypto/importKey
-    return crypto.subtle.importKey('raw', keyBytes, 'HKDF', false, [ 'deriveBits', 'deriveKey' ]);
+    return await crypto.subtle.importKey('raw', keyBytes, 'HKDF', false, [ 'deriveBits', 'deriveKey' ]);
 }
 
-;// CONCATENATED MODULE: ./modules/e2ee/Context.js
+;// ./modules/e2ee/Context.js
 /* eslint-disable no-bitwise */
 /* global BigInt */
 
@@ -392,8 +391,7 @@ class Context {
     }
 }
 
-;// CONCATENATED MODULE: ./modules/e2ee/Worker.js
-/* global TransformStream */
+;// ./modules/e2ee/Worker.js
 /* eslint-disable no-bitwise */
 
 // Worker for E2EE/Insertable streams.
@@ -445,7 +443,7 @@ function handleTransform(context, operation, readableStream, writableStream) {
     }
 }
 
-onmessage = async event => {
+onmessage = event => {
     const { operation } = event.data;
 
     if (operation === 'initialize') {
